/* this is little more than a thin wrapper around ndialog.h */

#include "core.h"
#include "simplestring.h"
#include "windowwrapper.h"
#include "nativedialog.h"

#if BUILD_SDL
#include "SDL2/SDL.h"
#endif

#include <stdlib.h>

using namespace NativeDialog;

static bool isInitialized = false;

/* SDL needs some simple state resets after a native dialog was opened.
 */
static void FixForSDL(Window *Win)
{
	#if BUILD_SDL
	/* return focus to our main window */
	if ( Win )
	{
		SDL_RaiseWindow( Win->GetSDLWindow() );
	}

	SDL_PumpEvents();

	SDL_SetRelativeMouseMode( SDL_TRUE );
 
	/* really ugly hack: when opening native dialogs in response to a keypress,
	 * SDL does not receive the SDL_KEYUP Events for those keys, as they are
	 * delivered to the dialog. In order to not have stuck keys in SDL afterwards,
	 * we can call this method to clear SDL's internal(!) keyboard state matrix
	 * to 0. Not pretty, but works...
	 */
	SDL_SetModState( KMOD_NONE );
	int NumKeys, Key;
	Uint8* SDLKeysState = (Uint8*) SDL_GetKeyboardState( &NumKeys );
	for ( Key = 0; Key < NumKeys; ++Key )
	{
		SDLKeysState[ Key ] = 0;
	}

	#endif
}

static const char** MakeCFilters( const Array<SimpleString>& Filters)
{
	const char** cfilt = 0;
	if ( Filters.Size() > 0 ) {
		cfilt = (const char**) calloc( Filters.Size() * 2 + 1, sizeof(char*) );
		int i = 0;
		FOR_EACH_ARRAY( Filt, Filters, SimpleString ) {
			cfilt[ i++ ] = (*Filt).CStr();
			cfilt[ i++ ] = (*Filt).CStr();
		}
		cfilt[i] = 0;
	}
	return cfilt;
}

namespace NativeDialog {

bool Init()
{
	if ( !isInitialized ) {
		isInitialized = ( ndialog_init() != 0 );
	}
	return isInitialized;
}

void Exit()
{
	if ( isInitialized ) {
		ndialog_exit();
	}
	isInitialized = false;
}

BtnType MessageBox( Window *Parent, const SimpleString& Title, const SimpleString& Message, const MsgType Msgtype, const DlgType Dlgtype )
{
	BtnType res = (BtnType) ndialog_messagebox( Title.CStr(), Message.CStr(), (ndialog_msgtype) Msgtype, (ndialog_dlgtype) Dlgtype );
	FixForSDL( Parent );
	return res;
}

SimpleString SaveFile( Window *Parent, const SimpleString Title, const SimpleString Path, const Array<SimpleString>& Filters, const bool AllowCreateFolder /*=true*/, const bool AskOverwrite  /*=true*/ )
{
	const char** cfilters = MakeCFilters( Filters );

	SimpleString res = ndialog_savefile( Title.CStr(), Path.CStr(), cfilters, AllowCreateFolder, AskOverwrite );
	if ( cfilters ) { free( cfilters ); }
	FixForSDL( Parent );
	return res;
}

SimpleString OpenFile( Window *Parent, const SimpleString Title, const SimpleString Path, const Array<SimpleString>& Filters, const bool AllowMultiple, int* NumRes )
{
	const char** cfilters = MakeCFilters( Filters );

	SimpleString res = ndialog_openfile( Title.CStr(), Path.CStr(), cfilters, AllowMultiple, NumRes );
	if ( cfilters ) { free( cfilters ); }
	FixForSDL( Parent );
	return res;
}

ErrType LastError()
{
	return (ErrType) ndialog_lasterror();
}

}
