#ifndef HASHREVERSER_H
#define HASHREVERSER_H

// Sequences range from 0 ... SEQUENCE_INDEX_CAP-1
#define SEQUENCE_INDEX_CAP 300

class HashReverser {
public:
	HashReverser( const char *InitialStrings, const char *SequencePatterns );
	~HashReverser();
	SimpleString Reverse( HashedString Hash );
	void RegisterHash( const HashedString& Hash, const SimpleString& String );

private:
	SimpleString CheckSequence( HashedString Hash );

	struct SequenceEntry
	{
		SimpleString m_Pattern;
		Array<bool> m_UsedIndices;
		int32 m_LastIndex;
		int32 m_HighestConsecutiveIndex;

		SequenceEntry()
		: m_Pattern( "" )
		, m_UsedIndices()
		, m_LastIndex( -1 )
		, m_HighestConsecutiveIndex( -1 )
		{}

		SequenceEntry( SimpleString Pattern )
		: m_Pattern( Pattern )
		, m_UsedIndices()
		, m_LastIndex( -1 )
		, m_HighestConsecutiveIndex( -1 )
		{}

		int32 UseIndex( int32 Index )
		{
			if ( Index >= SEQUENCE_INDEX_CAP )
			{
				WARN;
				return -1;
			}
			m_UsedIndices.Resize( Index + 1 );
			m_UsedIndices[ Index ] = true;
			if ( Index == m_HighestConsecutiveIndex + 1 )
			{
				m_HighestConsecutiveIndex = Index;
			}
			return Index;
		}

		int32 GetFirstIndex()
		{
			return m_HighestConsecutiveIndex + 1;
		}

		int32 GetNextIndex( int32 PrevIndex )
		{
			int32 NextIndex = PrevIndex + 1;
			while( NextIndex < SEQUENCE_INDEX_CAP && (uint32)NextIndex < m_UsedIndices.Size() && m_UsedIndices[NextIndex] )
			{
				++NextIndex;
			}
			if( NextIndex >= SEQUENCE_INDEX_CAP )
			{
				NextIndex = -1;
			}
			return NextIndex;
		}

		bool CheckHash( HashedString TestHash )
		{
			for( int32 i = GetFirstIndex(); i >= 0; i = GetNextIndex( i ) )
			{
				SimpleString SequenceName = SimpleString::PrintF( m_Pattern, i );
				HashedString SequenceHash = SequenceName;
				if ( TestHash == SequenceHash )
				{
					m_LastIndex = UseIndex( i );
					return true;
				}
			}
			m_LastIndex = -1;
			return false; // TODO
		}

		SimpleString GetLastValue()
		{
			if( m_LastIndex > -1 )
			{
				return SimpleString::PrintF( m_Pattern, m_LastIndex );
			}
			return "";
		}
	};

	Array<SequenceEntry> m_Sequences;
};



#endif // HASHREVERSER_H